@@ -62,7 +62,7 @@
 
 
 /* Copy the first part of user declarations.  */
-#line 38 "asn1parse.y" /* yacc.c:339  */
+#line 38 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:339  */
 
 
 #include <config.h>
@@ -314,7 +314,7 @@ extern int yydebug;
 
 union YYSTYPE
 {
-#line 72 "asn1parse.y" /* yacc.c:355  */
+#line 72 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:355  */
 
     int64_t constant;
     struct value *value;
@@ -1621,7 +1621,7 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 242 "asn1parse.y" /* yacc.c:1646  */
+#line 242 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			checkundefined();
 		}
@@ -1629,31 +1629,31 @@ yyreduce:
     break;
 
   case 3:
-#line 248 "asn1parse.y" /* yacc.c:1646  */
+#line 248 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     { default_tag_env = TE_EXPLICIT; }
 #line 1635 "asn1parse.c" /* yacc.c:1646  */
     break;
 
   case 4:
-#line 250 "asn1parse.y" /* yacc.c:1646  */
+#line 250 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     { default_tag_env = TE_IMPLICIT; }
 #line 1641 "asn1parse.c" /* yacc.c:1646  */
     break;
 
   case 5:
-#line 252 "asn1parse.y" /* yacc.c:1646  */
+#line 252 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     { lex_error_message("automatic tagging is not supported"); }
 #line 1647 "asn1parse.c" /* yacc.c:1646  */
     break;
 
   case 7:
-#line 257 "asn1parse.y" /* yacc.c:1646  */
+#line 257 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     { lex_error_message("no extensibility options supported"); }
 #line 1653 "asn1parse.c" /* yacc.c:1646  */
     break;
 
   case 17:
-#line 278 "asn1parse.y" /* yacc.c:1646  */
+#line 278 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		    struct string_list *sl;
 		    for(sl = (yyvsp[-3].sl); sl != NULL; sl = sl->next) {
@@ -1667,7 +1667,7 @@ yyreduce:
     break;
 
   case 18:
-#line 290 "asn1parse.y" /* yacc.c:1646  */
+#line 290 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		    struct string_list *sl;
 		    for(sl = (yyvsp[-1].sl); sl != NULL; sl = sl->next)
@@ -1677,7 +1677,7 @@ yyreduce:
     break;
 
   case 25:
-#line 308 "asn1parse.y" /* yacc.c:1646  */
+#line 308 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		    (yyval.sl) = emalloc(sizeof(*(yyval.sl)));
 		    (yyval.sl)->string = (yyvsp[-2].name);
@@ -1687,7 +1687,7 @@ yyreduce:
     break;
 
   case 26:
-#line 314 "asn1parse.y" /* yacc.c:1646  */
+#line 314 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		    (yyval.sl) = emalloc(sizeof(*(yyval.sl)));
 		    (yyval.sl)->string = (yyvsp[0].name);
@@ -1697,7 +1697,7 @@ yyreduce:
     break;
 
   case 27:
-#line 322 "asn1parse.y" /* yacc.c:1646  */
+#line 322 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		    Symbol *s = addsym ((yyvsp[-2].name));
 		    s->stype = Stype;
@@ -1709,7 +1709,7 @@ yyreduce:
     break;
 
   case 45:
-#line 353 "asn1parse.y" /* yacc.c:1646  */
+#line 353 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_Boolean,
 				     TE_EXPLICIT, new_type(TBoolean));
@@ -1718,7 +1718,7 @@ yyreduce:
     break;
 
   case 46:
-#line 360 "asn1parse.y" /* yacc.c:1646  */
+#line 360 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		    if((yyvsp[-3].value)->type != integervalue)
 			lex_error_message("Non-integer used in first part of range");
@@ -1732,7 +1732,7 @@ yyreduce:
     break;
 
   case 47:
-#line 370 "asn1parse.y" /* yacc.c:1646  */
+#line 370 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		    if((yyvsp[-3].value)->type != integervalue)
 			lex_error_message("Non-integer in first part of range");
@@ -1744,7 +1744,7 @@ yyreduce:
     break;
 
   case 48:
-#line 378 "asn1parse.y" /* yacc.c:1646  */
+#line 378 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		    if((yyvsp[-1].value)->type != integervalue)
 			lex_error_message("Non-integer in second part of range");
@@ -1756,7 +1756,7 @@ yyreduce:
     break;
 
   case 49:
-#line 386 "asn1parse.y" /* yacc.c:1646  */
+#line 386 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		    if((yyvsp[-1].value)->type != integervalue)
 			lex_error_message("Non-integer used in limit");
@@ -1768,7 +1768,7 @@ yyreduce:
     break;
 
   case 50:
-#line 397 "asn1parse.y" /* yacc.c:1646  */
+#line 397 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_Integer,
 				     TE_EXPLICIT, new_type(TInteger));
@@ -1777,7 +1777,7 @@ yyreduce:
     break;
 
   case 51:
-#line 402 "asn1parse.y" /* yacc.c:1646  */
+#line 402 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.type) = new_type(TInteger);
 			(yyval.type)->range = (yyvsp[0].range);
@@ -1787,7 +1787,7 @@ yyreduce:
     break;
 
   case 52:
-#line 408 "asn1parse.y" /* yacc.c:1646  */
+#line 408 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		  (yyval.type) = new_type(TInteger);
 		  (yyval.type)->members = (yyvsp[-1].members);
@@ -1797,7 +1797,7 @@ yyreduce:
     break;
 
   case 53:
-#line 416 "asn1parse.y" /* yacc.c:1646  */
+#line 416 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.members) = emalloc(sizeof(*(yyval.members)));
 			ASN1_TAILQ_INIT((yyval.members));
@@ -1807,7 +1807,7 @@ yyreduce:
     break;
 
   case 54:
-#line 422 "asn1parse.y" /* yacc.c:1646  */
+#line 422 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			ASN1_TAILQ_INSERT_TAIL((yyvsp[-2].members), (yyvsp[0].member), members);
 			(yyval.members) = (yyvsp[-2].members);
@@ -1816,13 +1816,13 @@ yyreduce:
     break;
 
   case 55:
-#line 427 "asn1parse.y" /* yacc.c:1646  */
+#line 427 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     { (yyval.members) = (yyvsp[-2].members); }
 #line 1822 "asn1parse.c" /* yacc.c:1646  */
     break;
 
   case 56:
-#line 431 "asn1parse.y" /* yacc.c:1646  */
+#line 431 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.member) = emalloc(sizeof(*(yyval.member)));
 			(yyval.member)->name = (yyvsp[-3].name);
@@ -1837,7 +1837,7 @@ yyreduce:
     break;
 
   case 57:
-#line 444 "asn1parse.y" /* yacc.c:1646  */
+#line 444 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		  (yyval.type) = new_type(TInteger);
 		  (yyval.type)->members = (yyvsp[-1].members);
@@ -1847,7 +1847,7 @@ yyreduce:
     break;
 
   case 59:
-#line 455 "asn1parse.y" /* yacc.c:1646  */
+#line 455 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		  (yyval.type) = new_type(TBitString);
 		  (yyval.type)->members = emalloc(sizeof(*(yyval.type)->members));
@@ -1858,7 +1858,7 @@ yyreduce:
     break;
 
   case 60:
-#line 462 "asn1parse.y" /* yacc.c:1646  */
+#line 462 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		  (yyval.type) = new_type(TBitString);
 		  (yyval.type)->members = (yyvsp[-1].members);
@@ -1868,7 +1868,7 @@ yyreduce:
     break;
 
   case 61:
-#line 470 "asn1parse.y" /* yacc.c:1646  */
+#line 470 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_OID,
 				     TE_EXPLICIT, new_type(TOID));
@@ -1877,7 +1877,7 @@ yyreduce:
     break;
 
   case 62:
-#line 476 "asn1parse.y" /* yacc.c:1646  */
+#line 476 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		    Type *t = new_type(TOctetString);
 		    t->range = (yyvsp[0].range);
@@ -1893,7 +1893,7 @@ yyreduce:
     break;
 
   case 63:
-#line 490 "asn1parse.y" /* yacc.c:1646  */
+#line 490 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_Null,
 				     TE_EXPLICIT, new_type(TNull));
@@ -1902,19 +1902,19 @@ yyreduce:
     break;
 
   case 64:
-#line 497 "asn1parse.y" /* yacc.c:1646  */
+#line 497 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     { (yyval.range) = NULL; }
 #line 1908 "asn1parse.c" /* yacc.c:1646  */
     break;
 
   case 65:
-#line 499 "asn1parse.y" /* yacc.c:1646  */
+#line 499 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     { (yyval.range) = (yyvsp[0].range); }
 #line 1914 "asn1parse.c" /* yacc.c:1646  */
     break;
 
   case 66:
-#line 504 "asn1parse.y" /* yacc.c:1646  */
+#line 504 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		  (yyval.type) = new_type(TSequence);
 		  (yyval.type)->members = (yyvsp[-1].members);
@@ -1924,7 +1924,7 @@ yyreduce:
     break;
 
   case 67:
-#line 510 "asn1parse.y" /* yacc.c:1646  */
+#line 510 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		  (yyval.type) = new_type(TSequence);
 		  (yyval.type)->members = NULL;
@@ -1934,7 +1934,7 @@ yyreduce:
     break;
 
   case 68:
-#line 518 "asn1parse.y" /* yacc.c:1646  */
+#line 518 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		  (yyval.type) = new_type(TSequenceOf);
 		  (yyval.type)->range = (yyvsp[-2].range);
@@ -1951,7 +1951,7 @@ yyreduce:
     break;
 
   case 69:
-#line 533 "asn1parse.y" /* yacc.c:1646  */
+#line 533 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		  (yyval.type) = new_type(TSet);
 		  (yyval.type)->members = (yyvsp[-1].members);
@@ -1961,7 +1961,7 @@ yyreduce:
     break;
 
   case 70:
-#line 539 "asn1parse.y" /* yacc.c:1646  */
+#line 539 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		  (yyval.type) = new_type(TSet);
 		  (yyval.type)->members = NULL;
@@ -1971,7 +1971,7 @@ yyreduce:
     break;
 
   case 71:
-#line 547 "asn1parse.y" /* yacc.c:1646  */
+#line 547 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		  (yyval.type) = new_type(TSetOf);
 		  (yyval.type)->subtype = (yyvsp[0].type);
@@ -1981,7 +1981,7 @@ yyreduce:
     break;
 
   case 72:
-#line 555 "asn1parse.y" /* yacc.c:1646  */
+#line 555 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		  (yyval.type) = new_type(TChoice);
 		  (yyval.type)->members = (yyvsp[-1].members);
@@ -1990,7 +1990,7 @@ yyreduce:
     break;
 
   case 75:
-#line 566 "asn1parse.y" /* yacc.c:1646  */
+#line 566 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		  Symbol *s = addsym((yyvsp[0].name));
 		  (yyval.type) = new_type(TType);
@@ -2003,7 +2003,7 @@ yyreduce:
     break;
 
   case 76:
-#line 577 "asn1parse.y" /* yacc.c:1646  */
+#line 577 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_GeneralizedTime,
 				     TE_EXPLICIT, new_type(TGeneralizedTime));
@@ -2012,7 +2012,7 @@ yyreduce:
     break;
 
   case 77:
-#line 582 "asn1parse.y" /* yacc.c:1646  */
+#line 582 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_UTCTime,
 				     TE_EXPLICIT, new_type(TUTCTime));
@@ -2021,7 +2021,7 @@ yyreduce:
     break;
 
   case 78:
-#line 589 "asn1parse.y" /* yacc.c:1646  */
+#line 589 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		    /* if (Constraint.type == contentConstrant) {
 		       assert(Constraint.u.constraint.type == octetstring|bitstring-w/o-NamedBitList); // remember to check type reference too
@@ -2038,7 +2038,7 @@ yyreduce:
     break;
 
   case 79:
-#line 605 "asn1parse.y" /* yacc.c:1646  */
+#line 605 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		    (yyval.constraint_spec) = (yyvsp[-1].constraint_spec);
 		}
@@ -2046,7 +2046,7 @@ yyreduce:
     break;
 
   case 83:
-#line 618 "asn1parse.y" /* yacc.c:1646  */
+#line 618 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		    (yyval.constraint_spec) = new_constraint_spec(CT_CONTENTS);
 		    (yyval.constraint_spec)->u.content.type = (yyvsp[0].type);
@@ -2056,7 +2056,7 @@ yyreduce:
     break;
 
   case 84:
-#line 624 "asn1parse.y" /* yacc.c:1646  */
+#line 624 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		    if ((yyvsp[0].value)->type != objectidentifiervalue)
 			lex_error_message("Non-OID used in ENCODED BY constraint");
@@ -2068,7 +2068,7 @@ yyreduce:
     break;
 
   case 85:
-#line 632 "asn1parse.y" /* yacc.c:1646  */
+#line 632 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		    if ((yyvsp[0].value)->type != objectidentifiervalue)
 			lex_error_message("Non-OID used in ENCODED BY constraint");
@@ -2080,7 +2080,7 @@ yyreduce:
     break;
 
   case 86:
-#line 642 "asn1parse.y" /* yacc.c:1646  */
+#line 642 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		    (yyval.constraint_spec) = new_constraint_spec(CT_USER);
 		}
@@ -2088,7 +2088,7 @@ yyreduce:
     break;
 
   case 87:
-#line 648 "asn1parse.y" /* yacc.c:1646  */
+#line 648 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.type) = new_type(TTag);
 			(yyval.type)->tag = (yyvsp[-2].tag);
@@ -2108,7 +2108,7 @@ yyreduce:
     break;
 
   case 88:
-#line 666 "asn1parse.y" /* yacc.c:1646  */
+#line 666 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.tag).tagclass = (yyvsp[-2].constant);
 			(yyval.tag).tagvalue = (yyvsp[-1].constant);
@@ -2118,7 +2118,7 @@ yyreduce:
     break;
 
   case 89:
-#line 674 "asn1parse.y" /* yacc.c:1646  */
+#line 674 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.constant) = ASN1_C_CONTEXT;
 		}
@@ -2126,7 +2126,7 @@ yyreduce:
     break;
 
   case 90:
-#line 678 "asn1parse.y" /* yacc.c:1646  */
+#line 678 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.constant) = ASN1_C_UNIV;
 		}
@@ -2134,7 +2134,7 @@ yyreduce:
     break;
 
   case 91:
-#line 682 "asn1parse.y" /* yacc.c:1646  */
+#line 682 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.constant) = ASN1_C_APPL;
 		}
@@ -2142,7 +2142,7 @@ yyreduce:
     break;
 
   case 92:
-#line 686 "asn1parse.y" /* yacc.c:1646  */
+#line 686 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.constant) = ASN1_C_PRIVATE;
 		}
@@ -2150,7 +2150,7 @@ yyreduce:
     break;
 
   case 93:
-#line 692 "asn1parse.y" /* yacc.c:1646  */
+#line 692 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.constant) = default_tag_env;
 		}
@@ -2158,7 +2158,7 @@ yyreduce:
     break;
 
   case 94:
-#line 696 "asn1parse.y" /* yacc.c:1646  */
+#line 696 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.constant) = default_tag_env;
 		}
@@ -2166,7 +2166,7 @@ yyreduce:
     break;
 
   case 95:
-#line 700 "asn1parse.y" /* yacc.c:1646  */
+#line 700 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.constant) = TE_IMPLICIT;
 		}
@@ -2174,7 +2174,7 @@ yyreduce:
     break;
 
   case 96:
-#line 707 "asn1parse.y" /* yacc.c:1646  */
+#line 707 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			Symbol *s;
 			s = addsym ((yyvsp[-3].name));
@@ -2187,7 +2187,7 @@ yyreduce:
     break;
 
   case 98:
-#line 721 "asn1parse.y" /* yacc.c:1646  */
+#line 721 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_GeneralString,
 				     TE_EXPLICIT, new_type(TGeneralString));
@@ -2196,7 +2196,7 @@ yyreduce:
     break;
 
   case 99:
-#line 726 "asn1parse.y" /* yacc.c:1646  */
+#line 726 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_TeletexString,
 				     TE_EXPLICIT, new_type(TTeletexString));
@@ -2205,7 +2205,7 @@ yyreduce:
     break;
 
   case 100:
-#line 731 "asn1parse.y" /* yacc.c:1646  */
+#line 731 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_UTF8String,
 				     TE_EXPLICIT, new_type(TUTF8String));
@@ -2214,7 +2214,7 @@ yyreduce:
     break;
 
   case 101:
-#line 736 "asn1parse.y" /* yacc.c:1646  */
+#line 736 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_PrintableString,
 				     TE_EXPLICIT, new_type(TPrintableString));
@@ -2223,7 +2223,7 @@ yyreduce:
     break;
 
   case 102:
-#line 741 "asn1parse.y" /* yacc.c:1646  */
+#line 741 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_VisibleString,
 				     TE_EXPLICIT, new_type(TVisibleString));
@@ -2232,7 +2232,7 @@ yyreduce:
     break;
 
   case 103:
-#line 746 "asn1parse.y" /* yacc.c:1646  */
+#line 746 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_IA5String,
 				     TE_EXPLICIT, new_type(TIA5String));
@@ -2241,7 +2241,7 @@ yyreduce:
     break;
 
   case 104:
-#line 751 "asn1parse.y" /* yacc.c:1646  */
+#line 751 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_BMPString,
 				     TE_EXPLICIT, new_type(TBMPString));
@@ -2250,7 +2250,7 @@ yyreduce:
     break;
 
   case 105:
-#line 756 "asn1parse.y" /* yacc.c:1646  */
+#line 756 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.type) = new_tag(ASN1_C_UNIV, UT_UniversalString,
 				     TE_EXPLICIT, new_type(TUniversalString));
@@ -2259,7 +2259,7 @@ yyreduce:
     break;
 
   case 106:
-#line 764 "asn1parse.y" /* yacc.c:1646  */
+#line 764 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.members) = emalloc(sizeof(*(yyval.members)));
 			ASN1_TAILQ_INIT((yyval.members));
@@ -2269,7 +2269,7 @@ yyreduce:
     break;
 
   case 107:
-#line 770 "asn1parse.y" /* yacc.c:1646  */
+#line 770 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			ASN1_TAILQ_INSERT_TAIL((yyvsp[-2].members), (yyvsp[0].member), members);
 			(yyval.members) = (yyvsp[-2].members);
@@ -2278,7 +2278,7 @@ yyreduce:
     break;
 
   case 108:
-#line 775 "asn1parse.y" /* yacc.c:1646  */
+#line 775 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		        struct member *m = ecalloc(1, sizeof(*m));
 			m->name = estrdup("...");
@@ -2291,7 +2291,7 @@ yyreduce:
     break;
 
   case 109:
-#line 786 "asn1parse.y" /* yacc.c:1646  */
+#line 786 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		  (yyval.member) = emalloc(sizeof(*(yyval.member)));
 		  (yyval.member)->name = (yyvsp[-1].name);
@@ -2304,7 +2304,7 @@ yyreduce:
     break;
 
   case 110:
-#line 797 "asn1parse.y" /* yacc.c:1646  */
+#line 797 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.member) = (yyvsp[0].member);
 			(yyval.member)->optional = 0;
@@ -2314,7 +2314,7 @@ yyreduce:
     break;
 
   case 111:
-#line 803 "asn1parse.y" /* yacc.c:1646  */
+#line 803 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.member) = (yyvsp[-1].member);
 			(yyval.member)->optional = 1;
@@ -2324,7 +2324,7 @@ yyreduce:
     break;
 
   case 112:
-#line 809 "asn1parse.y" /* yacc.c:1646  */
+#line 809 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.member) = (yyvsp[-2].member);
 			(yyval.member)->optional = 0;
@@ -2334,7 +2334,7 @@ yyreduce:
     break;
 
   case 113:
-#line 817 "asn1parse.y" /* yacc.c:1646  */
+#line 817 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.members) = emalloc(sizeof(*(yyval.members)));
 			ASN1_TAILQ_INIT((yyval.members));
@@ -2344,7 +2344,7 @@ yyreduce:
     break;
 
   case 114:
-#line 823 "asn1parse.y" /* yacc.c:1646  */
+#line 823 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			ASN1_TAILQ_INSERT_TAIL((yyvsp[-2].members), (yyvsp[0].member), members);
 			(yyval.members) = (yyvsp[-2].members);
@@ -2353,7 +2353,7 @@ yyreduce:
     break;
 
   case 115:
-#line 830 "asn1parse.y" /* yacc.c:1646  */
+#line 830 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		  (yyval.member) = emalloc(sizeof(*(yyval.member)));
 		  (yyval.member)->name = (yyvsp[-3].name);
@@ -2368,13 +2368,13 @@ yyreduce:
     break;
 
   case 117:
-#line 843 "asn1parse.y" /* yacc.c:1646  */
+#line 843 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     { (yyval.objid) = NULL; }
 #line 2374 "asn1parse.c" /* yacc.c:1646  */
     break;
 
   case 118:
-#line 847 "asn1parse.y" /* yacc.c:1646  */
+#line 847 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.objid) = (yyvsp[-1].objid);
 		}
@@ -2382,7 +2382,7 @@ yyreduce:
     break;
 
   case 119:
-#line 853 "asn1parse.y" /* yacc.c:1646  */
+#line 853 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.objid) = NULL;
 		}
@@ -2390,7 +2390,7 @@ yyreduce:
     break;
 
   case 120:
-#line 857 "asn1parse.y" /* yacc.c:1646  */
+#line 857 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		        if ((yyvsp[0].objid)) {
 				(yyval.objid) = (yyvsp[0].objid);
@@ -2403,7 +2403,7 @@ yyreduce:
     break;
 
   case 121:
-#line 868 "asn1parse.y" /* yacc.c:1646  */
+#line 868 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.objid) = new_objid((yyvsp[-3].name), (yyvsp[-1].constant));
 		}
@@ -2411,7 +2411,7 @@ yyreduce:
     break;
 
   case 122:
-#line 872 "asn1parse.y" /* yacc.c:1646  */
+#line 872 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		    Symbol *s = addsym((yyvsp[0].name));
 		    if(s->stype != SValue ||
@@ -2426,7 +2426,7 @@ yyreduce:
     break;
 
   case 123:
-#line 883 "asn1parse.y" /* yacc.c:1646  */
+#line 883 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		    (yyval.objid) = new_objid(NULL, (yyvsp[0].constant));
 		}
@@ -2434,7 +2434,7 @@ yyreduce:
     break;
 
   case 133:
-#line 906 "asn1parse.y" /* yacc.c:1646  */
+#line 906 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			Symbol *s = addsym((yyvsp[0].name));
 			if(s->stype != SValue)
@@ -2447,7 +2447,7 @@ yyreduce:
     break;
 
   case 134:
-#line 917 "asn1parse.y" /* yacc.c:1646  */
+#line 917 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.value) = emalloc(sizeof(*(yyval.value)));
 			(yyval.value)->type = stringvalue;
@@ -2457,7 +2457,7 @@ yyreduce:
     break;
 
   case 135:
-#line 925 "asn1parse.y" /* yacc.c:1646  */
+#line 925 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.value) = emalloc(sizeof(*(yyval.value)));
 			(yyval.value)->type = booleanvalue;
@@ -2467,7 +2467,7 @@ yyreduce:
     break;
 
   case 136:
-#line 931 "asn1parse.y" /* yacc.c:1646  */
+#line 931 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.value) = emalloc(sizeof(*(yyval.value)));
 			(yyval.value)->type = booleanvalue;
@@ -2477,7 +2477,7 @@ yyreduce:
     break;
 
   case 137:
-#line 939 "asn1parse.y" /* yacc.c:1646  */
+#line 939 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.value) = emalloc(sizeof(*(yyval.value)));
 			(yyval.value)->type = integervalue;
@@ -2487,14 +2487,14 @@ yyreduce:
     break;
 
   case 139:
-#line 950 "asn1parse.y" /* yacc.c:1646  */
+#line 950 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 		}
 #line 2494 "asn1parse.c" /* yacc.c:1646  */
     break;
 
   case 140:
-#line 955 "asn1parse.y" /* yacc.c:1646  */
+#line 955 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1646  */
     {
 			(yyval.value) = emalloc(sizeof(*(yyval.value)));
 			(yyval.value)->type = objectidentifiervalue;
@@ -2732,7 +2732,7 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 962 "asn1parse.y" /* yacc.c:1906  */
+#line 962 "/tmp/h2/lib/asn1/asn1parse.y" /* yacc.c:1906  */
 
 
 void
