@@ -62,7 +62,7 @@
 
 
 /* Copy the first part of user declarations.  */
-#line 1 "parse.y" /* yacc.c:339  */
+#line 1 "/tmp/h2/lib/com_err/parse.y" /* yacc.c:339  */
 
 /*
  * Copyright (c) 1998 - 2000 Kungliga Tekniska Högskolan
@@ -175,7 +175,7 @@ extern int yydebug;
 
 union YYSTYPE
 {
-#line 54 "parse.y" /* yacc.c:355  */
+#line 54 "/tmp/h2/lib/com_err/parse.y" /* yacc.c:355  */
 
   char *string;
   int number;
@@ -1276,7 +1276,7 @@ yyreduce:
   switch (yyn)
     {
         case 6:
-#line 74 "parse.y" /* yacc.c:1646  */
+#line 74 "/tmp/h2/lib/com_err/parse.y" /* yacc.c:1646  */
     {
 		    id_str = (yyvsp[0].string);
 		}
@@ -1284,7 +1284,7 @@ yyreduce:
     break;
 
   case 7:
-#line 80 "parse.y" /* yacc.c:1646  */
+#line 80 "/tmp/h2/lib/com_err/parse.y" /* yacc.c:1646  */
     {
 		    base_id = name2number((yyvsp[0].string));
 		    strlcpy(name, (yyvsp[0].string), sizeof(name));
@@ -1294,7 +1294,7 @@ yyreduce:
     break;
 
   case 8:
-#line 86 "parse.y" /* yacc.c:1646  */
+#line 86 "/tmp/h2/lib/com_err/parse.y" /* yacc.c:1646  */
     {
 		    base_id = name2number((yyvsp[-1].string));
 		    strlcpy(name, (yyvsp[0].string), sizeof(name));
@@ -1305,7 +1305,7 @@ yyreduce:
     break;
 
   case 11:
-#line 99 "parse.y" /* yacc.c:1646  */
+#line 99 "/tmp/h2/lib/com_err/parse.y" /* yacc.c:1646  */
     {
 			number = (yyvsp[0].number);
 		}
@@ -1313,7 +1313,7 @@ yyreduce:
     break;
 
   case 12:
-#line 103 "parse.y" /* yacc.c:1646  */
+#line 103 "/tmp/h2/lib/com_err/parse.y" /* yacc.c:1646  */
     {
 		    free(prefix);
 		    asprintf (&prefix, "%s_", (yyvsp[0].string));
@@ -1325,7 +1325,7 @@ yyreduce:
     break;
 
   case 13:
-#line 111 "parse.y" /* yacc.c:1646  */
+#line 111 "/tmp/h2/lib/com_err/parse.y" /* yacc.c:1646  */
     {
 		    prefix = realloc(prefix, 1);
 		    if (prefix == NULL)
@@ -1336,7 +1336,7 @@ yyreduce:
     break;
 
   case 14:
-#line 118 "parse.y" /* yacc.c:1646  */
+#line 118 "/tmp/h2/lib/com_err/parse.y" /* yacc.c:1646  */
     {
 		    struct error_code *ec = malloc(sizeof(*ec));
 
@@ -1360,7 +1360,7 @@ yyreduce:
     break;
 
   case 15:
-#line 138 "parse.y" /* yacc.c:1646  */
+#line 138 "/tmp/h2/lib/com_err/parse.y" /* yacc.c:1646  */
     {
 			YYACCEPT;
 		}
@@ -1596,7 +1596,7 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 143 "parse.y" /* yacc.c:1906  */
+#line 143 "/tmp/h2/lib/com_err/parse.y" /* yacc.c:1906  */
 
 
 static long
