@@ -62,7 +62,7 @@
 
 
 /* Copy the first part of user declarations.  */
-#line 34 "sel-gram.y" /* yacc.c:339  */
+#line 34 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:339  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -71,9 +71,21 @@
 #include <stdlib.h>
 #include <hx_locl.h>
 
+#if !defined(yylex)
+#define yylex   _hx509_sel_yylex
+#define yywrap  _hx509_sel_yywrap
+#endif
+#if !defined(yyparse)
+#define yyparse _hx509_sel_yyparse
+#define yyerror _hx509_sel_yyerror
+#define yylval  _hx509_sel_yylval
+#define yychar  _hx509_sel_yychar
+#define yydebug _hx509_sel_yydebug
+#define yynerrs _hx509_sel_yynerrs
+#endif
 
 
-#line 77 "sel-gram.c" /* yacc.c:339  */
+#line 89 "sel-gram.c" /* yacc.c:339  */
 
 # ifndef YY_NULLPTR
 #  if defined __cplusplus && 201103L <= __cplusplus
@@ -135,12 +147,12 @@ extern int yydebug;
 
 union YYSTYPE
 {
-#line 45 "sel-gram.y" /* yacc.c:355  */
+#line 57 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:355  */
 
     char *string;
     struct hx_expr *expr;
 
-#line 144 "sel-gram.c" /* yacc.c:355  */
+#line 156 "sel-gram.c" /* yacc.c:355  */
 };
 
 typedef union YYSTYPE YYSTYPE;
@@ -157,7 +169,7 @@ int yyparse (void);
 
 /* Copy the second part of user declarations.  */
 
-#line 161 "sel-gram.c" /* yacc.c:358  */
+#line 173 "sel-gram.c" /* yacc.c:358  */
 
 #ifdef short
 # undef short
@@ -455,9 +467,9 @@ static const yytype_uint8 yytranslate[] =
   /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_uint8 yyrline[] =
 {
-       0,    73,    73,    75,    76,    77,    78,    79,    80,    81,
-      84,    85,    88,    89,    90,    91,    92,    95,    96,    97,
-      98,   101,   102,   104,   107,   110,   112
+       0,    85,    85,    87,    88,    89,    90,    91,    92,    93,
+      96,    97,   100,   101,   102,   103,   104,   107,   108,   109,
+     110,   113,   114,   116,   119,   122,   124
 };
 #endif
 
@@ -1256,160 +1268,160 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 73 "sel-gram.y" /* yacc.c:1646  */
+#line 85 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
     { _hx509_expr_input.expr = (yyvsp[0].expr); }
-#line 1262 "sel-gram.c" /* yacc.c:1646  */
-    break;
-
-  case 3:
-#line 75 "sel-gram.y" /* yacc.c:1646  */
-    { (yyval.expr) = _hx509_make_expr(op_TRUE, NULL, NULL); }
-#line 1268 "sel-gram.c" /* yacc.c:1646  */
-    break;
-
-  case 4:
-#line 76 "sel-gram.y" /* yacc.c:1646  */
-    { (yyval.expr) = _hx509_make_expr(op_FALSE, NULL, NULL); }
 #line 1274 "sel-gram.c" /* yacc.c:1646  */
     break;
 
-  case 5:
-#line 77 "sel-gram.y" /* yacc.c:1646  */
-    { (yyval.expr) = _hx509_make_expr(op_NOT, (yyvsp[0].expr), NULL); }
+  case 3:
+#line 87 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
+    { (yyval.expr) = _hx509_make_expr(op_TRUE, NULL, NULL); }
 #line 1280 "sel-gram.c" /* yacc.c:1646  */
     break;
 
-  case 6:
-#line 78 "sel-gram.y" /* yacc.c:1646  */
-    { (yyval.expr) = _hx509_make_expr(op_AND, (yyvsp[-2].expr), (yyvsp[0].expr)); }
+  case 4:
+#line 88 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
+    { (yyval.expr) = _hx509_make_expr(op_FALSE, NULL, NULL); }
 #line 1286 "sel-gram.c" /* yacc.c:1646  */
     break;
 
-  case 7:
-#line 79 "sel-gram.y" /* yacc.c:1646  */
-    { (yyval.expr) = _hx509_make_expr(op_OR, (yyvsp[-2].expr), (yyvsp[0].expr)); }
+  case 5:
+#line 89 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
+    { (yyval.expr) = _hx509_make_expr(op_NOT, (yyvsp[0].expr), NULL); }
 #line 1292 "sel-gram.c" /* yacc.c:1646  */
     break;
 
-  case 8:
-#line 80 "sel-gram.y" /* yacc.c:1646  */
-    { (yyval.expr) = (yyvsp[-1].expr); }
+  case 6:
+#line 90 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
+    { (yyval.expr) = _hx509_make_expr(op_AND, (yyvsp[-2].expr), (yyvsp[0].expr)); }
 #line 1298 "sel-gram.c" /* yacc.c:1646  */
     break;
 
-  case 9:
-#line 81 "sel-gram.y" /* yacc.c:1646  */
-    { (yyval.expr) = _hx509_make_expr(op_COMP, (yyvsp[0].expr), NULL); }
+  case 7:
+#line 91 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
+    { (yyval.expr) = _hx509_make_expr(op_OR, (yyvsp[-2].expr), (yyvsp[0].expr)); }
 #line 1304 "sel-gram.c" /* yacc.c:1646  */
     break;
 
-  case 10:
-#line 84 "sel-gram.y" /* yacc.c:1646  */
-    { (yyval.expr) = _hx509_make_expr(expr_WORDS, (yyvsp[0].expr), NULL); }
+  case 8:
+#line 92 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
+    { (yyval.expr) = (yyvsp[-1].expr); }
 #line 1310 "sel-gram.c" /* yacc.c:1646  */
     break;
 
-  case 11:
-#line 85 "sel-gram.y" /* yacc.c:1646  */
-    { (yyval.expr) = _hx509_make_expr(expr_WORDS, (yyvsp[-2].expr), (yyvsp[0].expr)); }
+  case 9:
+#line 93 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
+    { (yyval.expr) = _hx509_make_expr(op_COMP, (yyvsp[0].expr), NULL); }
 #line 1316 "sel-gram.c" /* yacc.c:1646  */
     break;
 
-  case 12:
-#line 88 "sel-gram.y" /* yacc.c:1646  */
-    { (yyval.expr) = _hx509_make_expr(comp_EQ, (yyvsp[-3].expr), (yyvsp[0].expr)); }
+  case 10:
+#line 96 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
+    { (yyval.expr) = _hx509_make_expr(expr_WORDS, (yyvsp[0].expr), NULL); }
 #line 1322 "sel-gram.c" /* yacc.c:1646  */
     break;
 
-  case 13:
-#line 89 "sel-gram.y" /* yacc.c:1646  */
-    { (yyval.expr) = _hx509_make_expr(comp_NE, (yyvsp[-3].expr), (yyvsp[0].expr)); }
+  case 11:
+#line 97 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
+    { (yyval.expr) = _hx509_make_expr(expr_WORDS, (yyvsp[-2].expr), (yyvsp[0].expr)); }
 #line 1328 "sel-gram.c" /* yacc.c:1646  */
     break;
 
-  case 14:
-#line 90 "sel-gram.y" /* yacc.c:1646  */
-    { (yyval.expr) = _hx509_make_expr(comp_TAILEQ, (yyvsp[-2].expr), (yyvsp[0].expr)); }
+  case 12:
+#line 100 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
+    { (yyval.expr) = _hx509_make_expr(comp_EQ, (yyvsp[-3].expr), (yyvsp[0].expr)); }
 #line 1334 "sel-gram.c" /* yacc.c:1646  */
     break;
 
-  case 15:
-#line 91 "sel-gram.y" /* yacc.c:1646  */
-    { (yyval.expr) = _hx509_make_expr(comp_IN, (yyvsp[-4].expr), (yyvsp[-1].expr)); }
+  case 13:
+#line 101 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
+    { (yyval.expr) = _hx509_make_expr(comp_NE, (yyvsp[-3].expr), (yyvsp[0].expr)); }
 #line 1340 "sel-gram.c" /* yacc.c:1646  */
     break;
 
-  case 16:
-#line 92 "sel-gram.y" /* yacc.c:1646  */
-    { (yyval.expr) = _hx509_make_expr(comp_IN, (yyvsp[-2].expr), (yyvsp[0].expr)); }
+  case 14:
+#line 102 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
+    { (yyval.expr) = _hx509_make_expr(comp_TAILEQ, (yyvsp[-2].expr), (yyvsp[0].expr)); }
 #line 1346 "sel-gram.c" /* yacc.c:1646  */
     break;
 
-  case 17:
-#line 95 "sel-gram.y" /* yacc.c:1646  */
-    { (yyval.expr) = (yyvsp[0].expr); }
+  case 15:
+#line 103 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
+    { (yyval.expr) = _hx509_make_expr(comp_IN, (yyvsp[-4].expr), (yyvsp[-1].expr)); }
 #line 1352 "sel-gram.c" /* yacc.c:1646  */
     break;
 
-  case 18:
-#line 96 "sel-gram.y" /* yacc.c:1646  */
-    { (yyval.expr) = (yyvsp[0].expr); }
+  case 16:
+#line 104 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
+    { (yyval.expr) = _hx509_make_expr(comp_IN, (yyvsp[-2].expr), (yyvsp[0].expr)); }
 #line 1358 "sel-gram.c" /* yacc.c:1646  */
     break;
 
-  case 19:
-#line 97 "sel-gram.y" /* yacc.c:1646  */
+  case 17:
+#line 107 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
     { (yyval.expr) = (yyvsp[0].expr); }
 #line 1364 "sel-gram.c" /* yacc.c:1646  */
     break;
 
-  case 20:
-#line 98 "sel-gram.y" /* yacc.c:1646  */
+  case 18:
+#line 108 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
     { (yyval.expr) = (yyvsp[0].expr); }
 #line 1370 "sel-gram.c" /* yacc.c:1646  */
     break;
 
-  case 21:
-#line 101 "sel-gram.y" /* yacc.c:1646  */
-    { (yyval.expr) = _hx509_make_expr(expr_NUMBER, (yyvsp[0].string), NULL); }
+  case 19:
+#line 109 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
+    { (yyval.expr) = (yyvsp[0].expr); }
 #line 1376 "sel-gram.c" /* yacc.c:1646  */
     break;
 
-  case 22:
-#line 102 "sel-gram.y" /* yacc.c:1646  */
-    { (yyval.expr) = _hx509_make_expr(expr_STRING, (yyvsp[0].string), NULL); }
+  case 20:
+#line 110 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
+    { (yyval.expr) = (yyvsp[0].expr); }
 #line 1382 "sel-gram.c" /* yacc.c:1646  */
     break;
 
+  case 21:
+#line 113 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
+    { (yyval.expr) = _hx509_make_expr(expr_NUMBER, (yyvsp[0].string), NULL); }
+#line 1388 "sel-gram.c" /* yacc.c:1646  */
+    break;
+
+  case 22:
+#line 114 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
+    { (yyval.expr) = _hx509_make_expr(expr_STRING, (yyvsp[0].string), NULL); }
+#line 1394 "sel-gram.c" /* yacc.c:1646  */
+    break;
+
   case 23:
-#line 104 "sel-gram.y" /* yacc.c:1646  */
+#line 116 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
     {
 			(yyval.expr) = _hx509_make_expr(expr_FUNCTION, (yyvsp[-3].string), (yyvsp[-1].expr)); }
-#line 1389 "sel-gram.c" /* yacc.c:1646  */
+#line 1401 "sel-gram.c" /* yacc.c:1646  */
     break;
 
   case 24:
-#line 107 "sel-gram.y" /* yacc.c:1646  */
+#line 119 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
     { (yyval.expr) = (yyvsp[-1].expr); }
-#line 1395 "sel-gram.c" /* yacc.c:1646  */
+#line 1407 "sel-gram.c" /* yacc.c:1646  */
     break;
 
   case 25:
-#line 110 "sel-gram.y" /* yacc.c:1646  */
+#line 122 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
     {
 			(yyval.expr) = _hx509_make_expr(expr_VAR, (yyvsp[-2].string), (yyvsp[0].expr)); }
-#line 1402 "sel-gram.c" /* yacc.c:1646  */
+#line 1414 "sel-gram.c" /* yacc.c:1646  */
     break;
 
   case 26:
-#line 112 "sel-gram.y" /* yacc.c:1646  */
+#line 124 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1646  */
     {
 			(yyval.expr) = _hx509_make_expr(expr_VAR, (yyvsp[0].string), NULL); }
-#line 1409 "sel-gram.c" /* yacc.c:1646  */
+#line 1421 "sel-gram.c" /* yacc.c:1646  */
     break;
 
 
-#line 1413 "sel-gram.c" /* yacc.c:1646  */
+#line 1425 "sel-gram.c" /* yacc.c:1646  */
       default: break;
     }
   /* User semantic actions sometimes alter yychar, and that requires
