@@ -72,7 +72,7 @@ extern int yydebug;
 
 union YYSTYPE
 {
-#line 45 "sel-gram.y" /* yacc.c:1909  */
+#line 57 "/tmp/h2/lib/hx509/sel-gram.y" /* yacc.c:1909  */
 
     char *string;
     struct hx_expr *expr;
