@@ -62,7 +62,7 @@
 
 
 /* Copy the first part of user declarations.  */
-#line 1 "slc-gram.y" /* yacc.c:339  */
+#line 1 "/tmp/h2/lib/sl/slc-gram.y" /* yacc.c:339  */
 
 /*
  * Copyright (c) 2004-2006 Kungliga Tekniska Högskolan
@@ -166,7 +166,7 @@ extern int yydebug;
 
 union YYSTYPE
 {
-#line 57 "slc-gram.y" /* yacc.c:355  */
+#line 57 "/tmp/h2/lib/sl/slc-gram.y" /* yacc.c:355  */
 
 	char *string;
 	struct assignment *assignment;
@@ -1254,7 +1254,7 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 71 "slc-gram.y" /* yacc.c:1646  */
+#line 71 "/tmp/h2/lib/sl/slc-gram.y" /* yacc.c:1646  */
     {
 			assignment = (yyvsp[0].assignment);
 		}
@@ -1262,7 +1262,7 @@ yyreduce:
     break;
 
   case 3:
-#line 77 "slc-gram.y" /* yacc.c:1646  */
+#line 77 "/tmp/h2/lib/sl/slc-gram.y" /* yacc.c:1646  */
     {
 			(yyvsp[-1].assignment)->next = (yyvsp[0].assignment);
 			(yyval.assignment) = (yyvsp[-1].assignment);
@@ -1271,7 +1271,7 @@ yyreduce:
     break;
 
   case 5:
-#line 85 "slc-gram.y" /* yacc.c:1646  */
+#line 85 "/tmp/h2/lib/sl/slc-gram.y" /* yacc.c:1646  */
     {
 			(yyval.assignment) = malloc(sizeof(*(yyval.assignment)));
 			(yyval.assignment)->name = (yyvsp[-2].string);
@@ -1284,7 +1284,7 @@ yyreduce:
     break;
 
   case 6:
-#line 94 "slc-gram.y" /* yacc.c:1646  */
+#line 94 "/tmp/h2/lib/sl/slc-gram.y" /* yacc.c:1646  */
     {
 			(yyval.assignment) = malloc(sizeof(*(yyval.assignment)));
 			(yyval.assignment)->name = (yyvsp[-4].string);
@@ -1525,7 +1525,7 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 104 "slc-gram.y" /* yacc.c:1906  */
+#line 104 "/tmp/h2/lib/sl/slc-gram.y" /* yacc.c:1906  */
 
 char *filename;
 FILE *cfile, *hfile;
